<?php

namespace Drupal\paragraphs_table\Form;

use Drupal\Core\Entity\Entity\EntityFormDisplay;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Entity\RevisionableEntityBundleInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\paragraphs\ParagraphInterface;
use Drupal\Core\Url;

class ActionButtonForm extends FormBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Construct a new ParagraphLineageRevisioner object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   Provides  service.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager) {
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
    );
  }

  public function getFormId() {
    return 'paragraphs_table_add';
  }

  public function buildForm(array $form, FormStateInterface $form_state, EntityInterface $entity = NULL, $field_name = NULL, $targetBundle = NULL) {

    $form_mode = 'default';
    $form_state->set('field_name', $field_name);
    if (!$form_state->has('entity')) {
      $form_state->set('entity', $entity);
    }
    $form_display = $form_state->get('form_display');
    if(empty($form_display)){
      $display = EntityFormDisplay::collectRenderDisplay($entity, $form_mode);
      foreach ($display->getComponents() as $name => $options) {
        if ($name != $field_name) {
          $display->removeComponent($name);
        }else{
          if($options['type'] != 'paragraphs_table_widget'){
            $display->removeComponent($name);
          }
        }
      }
      if(empty($display->getComponent($field_name))){
        $definitions = \Drupal::service('entity_field.manager')->getFieldDefinitions($entity->getEntityTypeId(), $entity->bundle());
        $options = $definitions[$field_name]->getDisplayOptions('form');
        $options['region'] = 'content';
        $options['type'] = 'paragraphs_table_widget';
        $display->setComponent($field_name, $options);
      }
      $form_state->set('form_display', $display);
      $form_display = $form_state->get('form_display');
    }
    $widget_state = WidgetBase::getWidgetState([], $field_name, $form_state);
    if (!empty($widget_state["paragraphs"]) && $widget_state["paragraphs"][0]["mode"] != 'edit') {
      $form_state_storage = $form_state->getStorage();
      $form_state_storage["field_storage"]["#parents"]["#fields"][$field_name]["paragraphs"][0]["mode"] = 'edit';
      $form_state->setStorage($form_state_storage);
    }
    $form_display->buildForm($entity, $form, $form_state);
    $triggering_input = $form_state->getUserInput();
    $triggering_element = $form_state->getTriggeringElement();
    $trigger_mode = FALSE;
    if(!empty($triggering_element) && !empty($triggering_element["#name"])){
      $temp = explode("_",$triggering_element["#name"]);
      $trigger_mode = end($temp);
    }
    // Add the field form.
    if (!empty($form[$field_name]["widget"]["add_more"])) {
      unset($form[$field_name]["widget"]["add_more"]["#suffix"]);
      $child = implode('_',['add_more','button',$targetBundle]);
      if (!empty($form[$field_name]["widget"]["add_more"][$child])) {
        unset($form[$field_name]["widget"]["add_more"][$child]["#attributes"]["class"]);
      }
      $edit_btn = 'edit_'.$field_name;
      $triggering_btn_add = FALSE;
      $triggering_btn_edit = FALSE;
      if(!empty($triggering_input)){
        if(!empty($triggering_input["_triggering_element_name"]) && $triggering_input["_triggering_element_name"] == implode('_',[$field_name,$targetBundle,'add_more'])){
          $triggering_btn_add = TRUE;
        }
        if(!empty($triggering_input[$edit_btn]) || in_array($trigger_mode, ['remove','duplicate'])){
          $triggering_btn_edit = TRUE;
        }
      }

      $form['#disable_inline_form_errors'] = TRUE;
      $form['#rebuild'] = TRUE;
      $form[$field_name]["widget"]['#no_header'] = true;
      foreach ($entity->get($field_name)->getValue() as $child => $value) {
        if (!empty($value) && !$triggering_btn_edit ) {
          $form[$field_name]["widget"][$child]["#access"] = FALSE;
        }
      }

      if(!$triggering_btn_add ){
        $paragraphsType = \Drupal::service('entity.manager')->getBundleInfo('paragraph');
        $paragraphsLabel = $paragraphsType[$targetBundle]['label'];
        $form[$field_name]["widget"]["add_more"]['edit'] = [
          '#type' => 'submit',
          '#name' => $edit_btn,
          '#value' => $this->t('Edit %label',['%label' => $paragraphsLabel]),
          '#access' => TRUE
        ];
        if (!empty($triggering_input) && $triggering_btn_edit){
          $form[$field_name]["widget"]["add_more"]['edit']['#access'] = FALSE;
        }
      }

      if($triggering_btn_edit || $triggering_btn_add){
        $form[$field_name]["widget"]["add_more"]['submit'] = [
          '#type' => 'submit',
          '#value' => $this->t('Save'),
        ];
      }

    }
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $entity = $this->buildEntity($form, $form_state);
    $form_state->get('form_display')->validateFormValues($entity, $form, $form_state);
  }

  /**
   * {@inheritDoc}
   * @see \Drupal\Core\Form\FormInterface::submitForm()
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $entity = $this->buildEntity($form, $form_state);
    $form_state->set('entity', $entity);
    $entity->setNewRevision(FALSE);
    $entity->save();
    $form_state->disableRedirect(FALSE);
    $form_state->setRedirectUrl(  Url::fromRoute('<current>'));
  }

  /**
   * Returns a cloned entity containing updated field values.
   *
   * Calling code may then validate the returned entity, and if valid, transfer
   * it back to the form state and save it.
   */
  protected function buildEntity(array $form, FormStateInterface $form_state) {
    /** @var $entity \Drupal\Core\Entity\EntityInterface */
    $entity = clone $form_state->get('entity');
    $field_name = $form_state->get('field_name');

    $form_state->get('form_display')
      ->extractFormValues($entity, $form, $form_state);
    return $entity;
  }
  /**
   * Gets the field the paragraph is referenced from.
   *
   * @param \Drupal\paragraphs\ParagraphInterface $paragraph
   *   Paragraph data.
   *
   * @return \Drupal\entity_reference_revisions\EntityReferenceRevisionsFieldItemList|null
   *   Parent field.
   */
  public function getParentField(ParagraphInterface $paragraph) {
    $parent = $paragraph->getParentEntity();

    if (!$parent) {
      return NULL;
    }

    return $parent->get($paragraph->get('parent_field_name')->value);
  }
  /**
   * Checks if a given entity should be saved as a new revision by default.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity to check for default new revision.
   *
   * @return bool
   *   TRUE if this entity is set to be saved as a new revision by default,
   *   FALSE otherwise.
   */
  protected function shouldCreateNewRevision(EntityInterface $entity) {
    $new_revision_default = FALSE;

    if ($bundle_entity_type = $entity->getEntityType()->getBundleEntityType()) {
      $bundle_entity = $this->entityTypeManager
        ->getStorage($bundle_entity_type)
        ->load($entity->bundle());

      if ($bundle_entity instanceof RevisionableEntityBundleInterface) {
        $new_revision_default = $bundle_entity->shouldCreateNewRevision();
      }
    }

    return $new_revision_default;
  }

  /**
   * Saves all of given entity's lineage as new revisions.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *   The entity whose lineage to save as new revisions.
   *
   * @return int
   *   Either SAVED_NEW or SAVED_UPDATED, depending on the operation performed.
   */
  public function saveNewRevision(ContentEntityInterface $entity) {
    $result = $this->doSaveNewRevision($entity);

    while ($entity instanceof ParagraphInterface) {
      $parent = $entity->getParentEntity();
      if (!$parent) {
        break;
      }

      $this->doSaveNewRevision($parent);

      $entity = $parent;
    }

    return $result;
  }

  /**
   * Saves an entity as a new revision.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *   The entity to save.
   *
   * @return int
   *   Either SAVED_NEW or SAVED_UPDATED, depending on the operation performed.
   */
  protected function doSaveNewRevision(ContentEntityInterface $entity) {
    // Get the parent field item before saving, as after saving the
    // revision ID will be changed.
    if ($entity instanceof ParagraphInterface) {
      $parent_field_item = $this->getParentField($entity);
    }

    try {
      $entity->setNewRevision();
    } catch (\LogicException $e) {
      // A content entity not necessarily supports revisioning.
    }

    $status = $entity->save();

    if (isset($parent_field_item)) {
      $parent_field_item->set('target_revision_id', $entity->getRevisionId());
    }

    return $status;
  }
}
